<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" ?>
<xsl:stylesheet version="1.0" 
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
		xmlns="http://www.w3.org/1999/xhtml"
		xmlns:tsx="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat" 
		xmlns:md="http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat"
		xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
		xmlns:xsd="http://www.w3.org/2001/XMLSchema" 
		xmlns:dc="http://purl.org/dc/elements/1.1/"
		xsi:schemaLocation="http://www.environment-agency.gov.uk/XMLSchemas/EATimeSeriesDataExchangeFormat EATimeSeriesDataExchangeFormat.xsd http://www.environment-agency.gov.uk/XMLSchemas/EAMetadataFormat EAMetadata.xsd">

<!-- =================================== -->
<!-- Match if 'True' ...                 -->

	<xsl:variable name="matchComments" select="'False'" />
	<xsl:variable name="sortSetofValues" select="'True'" />

<!-- =================================== -->

	<dc:Title>Standard CSV mark-up (v2.2)</dc:Title>
	<dc:Date>2008-03-12</dc:Date>
	<dc:Description>Based on Standard XHTML mark-up (v3.0). Transform from Environment Agency Time-Series Data-Exchange Standard (TSX) documents (version 1.1 to 1.2) into comma seperated values (in an XHTML v1.0 wrapper).</dc:Description>
	<dc:Relation>File "EATimeSeriesDataExchangeFormat.xsd" must be in the same directory as the XML document.</dc:Relation>
	<dc:Creator>Chris Beales</dc:Creator>
	<dc:Format>text/xsl</dc:Format>
	<dc:Identifier>http://websites.chrisbeales.net/TSX/tsx-Trans_EA Standard CSV text.xsl</dc:Identifier>
	<dc:Language>en</dc:Language>
	<dc:Publisher>Chris Beales, Environment Agency</dc:Publisher>
	<dc:Rights>Copyright 2006-08 Environment Agency</dc:Rights>
	<dc:Type>XSL Stylesheet</dc:Type>
	<dc:Subject>XML, EATimeSeriesDataExchangeFormat, TSX, XHTML, HTML, CSV</dc:Subject>

<!-- =================================== -->

	<xsl:output 
		method="xml" 
		version="1.0" 
		encoding="iso-8859-1" 
		indent="no" 
		omit-xml-declaration="no"
		doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
		doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
		media-type="text/html" />

	<xsl:key name="kComment" match="//tsx:Comment" use="''" />

	<xsl:template match="/">
		<xsl:element name="html">
			<xsl:attribute name="xml:lang">en</xsl:attribute>
			<xsl:attribute name="lang">en</xsl:attribute>

			<xsl:element name="head">
				<xsl:element name="title">Time-series data file</xsl:element>
			</xsl:element>  <!-- head -->

			<xsl:element name="body">
				<xsl:element name="pre">		<!-- preserve -->
				<!-- Header -->
<xsl:text
>Environment Agency Time-Series Data Standard Document...

</xsl:text>

				<!-- Main -->

					<xsl:apply-templates select="tsx:EATimeSeriesDataExchangeFormat"/>

				<!-- Footer -->

<xsl:text>
Meanings for data/quality flags

</xsl:text>

					<xsl:for-each select="document('EATimeSeriesDataExchangeFormat.xsd')/xsd:schema/xsd:simpleType[@name ='DataQualityFlagType']/xsd:annotation/xsd:documentation/*/*">

						<xsl:value-of select="concat(@flagNumber, ', ', @flagName, ', ', ., '&#xd;&#xA;')"/>

					</xsl:for-each>

					<xsl:variable name="XSLfileName" select="document('')/*/dc:Title" />

					<xsl:value-of select="concat('&#xd;&#xA; &#xd;&#xA;This XHTML document has been created using the {', $XSLfileName, '} stylesheet. &#xd;&#xA;The XML data file is an Environment Agency Time-Series Data-Exchange Standard (TSDXS) document. &#xd;&#xA;For more information about the Standard, please refer to the Plain English Document located at ')" />
					<xsl:variable name="EAschemas">http://www.environment-agency.gov.uk/xmlschemas/</xsl:variable>
					<xsl:value-of select="$EAschemas" />
		
				</xsl:element>  <!-- pre -->
			</xsl:element>  <!-- body -->
		</xsl:element>  <!-- html -->		
	</xsl:template>

<!-- Main document -->
	<xsl:template match="tsx:EATimeSeriesDataExchangeFormat">
		<xsl:value-of select="concat('Publisher,', md:Publisher, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Source,', md:Source, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Description,', md:Description, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Creator,', md:Creator, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Date,', substring(md:Date, 9), '/', substring(md:Date, 6, 2), '/', substring(md:Date, 1, 4), '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Time,', md:Time, '&#xd;&#xA;')" />
		<xsl:value-of select="concat('Identifier,', md:Identifier, '&#xd;&#xA;')" />

		<xsl:choose>
			<xsl:when test="md:Publisher='Environment Agency'">
				<xsl:text>
Disclaimer: "The information provided is based on that currently available to the Agency. 
The Agency and its officers accept no liability whatsoever for any loss or damage arising 
from the interpretation or use of the information." &#xd;&#xA;</xsl:text>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="'&#xd;&#xA;&#xd;&#xA;&#xd;&#xA;&#xd;&#xA;'" />
			</xsl:otherwise>
		</xsl:choose>

<xsl:text>
========================================================

</xsl:text>

		<xsl:apply-templates select="tsx:Station" />

	</xsl:template>

<!-- Station element -->
	<xsl:template match="tsx:Station">
		<xsl:apply-templates select="tsx:SetofValues"/>
	</xsl:template>

	<xsl:template match="tsx:SetofValues">
		<xsl:value-of select="concat('Station Reference,', ../@stationReference, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Station Name,', ../@stationName, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Region,', ../@region, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('National Grid Reference,', ../@ngr, '&#xd;&#xA;&#xd;&#xA;')"/>

		<xsl:value-of select="concat('Parameter,', @parameter, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Qualifier,', @qualifier, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Data Type,', @dataType, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Period,', @period, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Interval,', @interval, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Characteristic,', @characteristic, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Product Reference,', @productRef, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Point Reference,', @pointReference, '&#xd;&#xA;')"/>
		<xsl:value-of select="concat('Units,', @units, '&#xd;&#xA;')"/>

		<xsl:variable name="vStartTime">
			<xsl:if test="@startTime">
				<xsl:value-of select="concat(' ', @startTime)" />
			</xsl:if>
		</xsl:variable>
		<xsl:value-of select="concat('Start Date,', substring(@startDate, 9), '/', substring(@startDate, 6, 2), '/', substring(@startDate, 1, 4), $vStartTime, '&#xd;&#xA;')"/>

		<xsl:variable name="vEndTime">
			<xsl:if test="@endTime">
				<xsl:value-of select="concat(' ', @endTime)" />
			</xsl:if>
		</xsl:variable>	
		<xsl:value-of select="concat('End Date,', substring(@endDate, 9), '/', substring(@endDate, 6, 2), '/', substring(@endDate, 1, 4), $vEndTime, '&#xd;&#xA;')"/>

		<xsl:value-of select="concat('Day Origin,', @dayOrigin, '&#xd;&#xA;')"/>

		<xsl:text>&#xd;&#xA;Date-time, Value, Quality, Data Flags, Comments&#xd;&#xA;</xsl:text>
		
		<xsl:choose>
			<xsl:when test="$sortSetofValues = 'True'">
				<xsl:apply-templates select="tsx:Value">
					<xsl:sort select="concat(@date, ' ',  @time)" order="ascending" data-type="text" />      	
				</xsl:apply-templates>
			</xsl:when>

			<xsl:otherwise>
				<xsl:apply-templates select="tsx:Value" />
			</xsl:otherwise>
		</xsl:choose>

		<xsl:if test="count(./tsx:Comment) != 0">
			<xsl:text>&#xd;&#xA;List of comments&#xd;&#xA;</xsl:text>
		</xsl:if>

		<xsl:choose>
			<xsl:when test="$sortSetofValues = 'True'">
				<xsl:apply-templates select="tsx:Comment">
					<xsl:sort select="concat(@startDate, ' ',  @startTime, ' ', @endDate, ' ',  @endTime)" order="ascending" data-type="text" />      	
				</xsl:apply-templates>
			</xsl:when>

			<xsl:otherwise>
				<xsl:apply-templates select="tsx:Comment" />
			</xsl:otherwise>
		</xsl:choose>

		<xsl:text>&#xd;&#xA;&#xd;&#xA;</xsl:text>
	</xsl:template>

<!-- Value element -->
	<xsl:template match="tsx:Value">
		<xsl:variable name="qualityFlag">
			<xsl:choose>
				<xsl:when test="@flag1=1">Good</xsl:when>
				<xsl:when test="@flag1=2">Suspect</xsl:when>
				<xsl:when test="@flag1=4">Unchecked</xsl:when>
				<xsl:when test="@flag1=5">Missing</xsl:when>
			</xsl:choose>
		</xsl:variable>

		<xsl:variable name="otherFlags">
			<xsl:if test="@flag1">
				<xsl:value-of select="@flag1"/>
				<xsl:if test="@percentFlag1">
					<xsl:value-of select="concat(' (', @percentFlag1, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag2">
				<xsl:value-of select="@flag2"/>
				<xsl:if test="@percentFlag2">
					<xsl:value-of select="concat(' (', @percentFlag2, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag3">
				<xsl:value-of select="@flag3"/>
				<xsl:if test="@percentFlag3">
					<xsl:value-of select="concat(' (', @percentFlag3, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag4">
				<xsl:value-of select="@flag4"/>
				<xsl:if test="@percentFlag4">
					<xsl:value-of select="concat(' (', @percentFlag4, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag5">
				<xsl:value-of select="@flag5"/>
				<xsl:if test="@percentFlag5">
					<xsl:value-of select="concat(' (', @percentFlag5, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag6">
				<xsl:value-of select="@flag6"/>
				<xsl:if test="@percentFlag6">
					<xsl:value-of select="concat(' (', @percentFlag6, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag7">
				<xsl:value-of select="@flag7"/>
				<xsl:if test="@percentFlag7">
					<xsl:value-of select="concat(' (', @percentFlag7, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag8">
				<xsl:value-of select="@flag8"/>
				<xsl:if test="@percentFlag8">
					<xsl:value-of select="concat(' (', @percentFlag8, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag9">
				<xsl:value-of select="@flag9"/>
				<xsl:if test="@percentFlag9">
					<xsl:value-of select="concat(' (', @percentFlag9, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>

			<xsl:if test="@flag10">
				<xsl:value-of select="@flag10"/>
				<xsl:if test="@percentFlag10">
					<xsl:value-of select="concat(' (', @percentFlag10, '%)')"/>
				</xsl:if>
				<xsl:value-of select="' '"/>
			</xsl:if>
		</xsl:variable>

	<!-- Output -->
		<xsl:value-of select="concat(substring(@date, 9), '/', substring(@date, 6, 2), '/', substring(@date, 1, 4))" />		
		<xsl:if test="@time">
			<xsl:value-of select="concat(' ', @time)"/>
		</xsl:if>
		<xsl:value-of select="','"/>

		<xsl:value-of select="concat(., ',')"/>

		<xsl:value-of select="concat($qualityFlag, ',')"/>

		<xsl:if test="$otherFlags != ''">
			<xsl:value-of select="concat(substring($otherFlags, 1, (string-length($otherFlags) - 1)), ',')"/>
		</xsl:if>

		<xsl:if test="$matchComments = 'True'">
			<xsl:variable name="testDayO">
				<xsl:value-of select="concat(substring(../@dayOrigin,1,2), substring(../@dayOrigin,4,2), substring(../@dayOrigin,7,2))" />
			</xsl:variable>

			<xsl:variable name="timeTest">
				<xsl:choose>
					<xsl:when test="@time">
						 <xsl:value-of select="concat(substring(@date,1,4), substring(@date,6,2), substring(@date,9,2), '.', substring(@time,1,2), substring(@time,4,2), substring(@time,7,2))" />
					</xsl:when>

					<xsl:when test="(../@dayOrigin) and not(@time)">					
						<xsl:value-of select="concat(substring(@date,1,4), substring(@date,6,2), substring(@date,9,2), '.', $testDayO)" />
					</xsl:when>				

					<xsl:otherwise>
						<xsl:value-of select="concat(substring(@date,1,4), substring(@date,6,2), substring(@date,9,2))" />
					</xsl:otherwise>
				</xsl:choose>
			</xsl:variable>

			<xsl:variable name="SoV" select="ancestor::*[1]" />

			<xsl:apply-templates select="key('kComment', '')[(substring(concat(substring(@startDate,1,4), substring(@startDate,6,2), substring(@startDate,9,2), '.', substring(@startTime,1,2), substring(@startTime,4,2), substring(@startTime,7,2), $testDayO), 1, 15) &lt;= $timeTest) and (substring(concat(substring(@endDate,1,4), substring(@endDate,6,2), substring(@endDate,9,2), '.', substring(@endTime,1,2), substring(@endTime,4,2), substring(@endTime,7,2), $testDayO), 1, 15) &gt;= $timeTest) and ($SoV = ancestor::*[1])]" mode="InTable" />
		</xsl:if>

		<xsl:text>&#xd;&#xA;</xsl:text>
	</xsl:template>
		
	<xsl:template match="tsx:Comment" mode="InTable">
		<xsl:value-of select="." />
		<xsl:if test="position() != last()"><xsl:text> | </xsl:text></xsl:if>
	</xsl:template>

<!-- Comment list at end -->
	<xsl:template match="tsx:Comment">
		<xsl:variable name="sDate">
			<xsl:if test="@startDate or @startTime">
				<xsl:value-of select="concat('(Start: ', substring(@startDate, 9), '/', substring(@startDate, 6, 2), '/', substring(@startDate, 1, 4))"/> 
				<xsl:if test="@startTime">
					<xsl:value-of select="concat(' ', @startTime)"/>
				</xsl:if>
				<xsl:text>) </xsl:text>
			</xsl:if>
		</xsl:variable>
			
		<xsl:variable name="eDate">
			<xsl:if test="@endDate or @endTime">
				<xsl:value-of select="concat('(End: ', substring(@endDate, 9), '/', substring(@endDate, 6, 2), '/', substring(@endDate, 1, 4))"/> 
				<xsl:if test="@endTime">
					<xsl:value-of select="concat(' ', @endTime)"/>
				</xsl:if>
				<xsl:text>) </xsl:text>
			</xsl:if>
		</xsl:variable>
		
		<xsl:value-of select="concat($sDate, $eDate, ., '&#xd;&#xA;')"/>
	</xsl:template>

</xsl:stylesheet>
